<?php if (!defined('PmWiki')) exit();
/**
 * precode.php
 * @author V.Krishn (vkrishn@insteps.net), Copyright 2012-2023
 * @note This file is part of Cookbook recipe "Precode" for PmWiki;
 * You can redistribute it and/or modify it under the terms of the
 * GNU General Public License as published by the  Free Software
 * Foundation; either version 2 of the License, or (at your option)
 * any later version. @see pmwiki.php for full details.
 *
 * @category pmwiki/cookbook
 * @package precode
 * @copyright Copyright (c) 2012-2023 V.Krishn (vkrishn@insteps.net)
 * @license GPL
 * @version 2.2.1
 */

global $Precode;
if ( ! IsEnabled($EnablePrecode, 0)) return;

	## Current script variables
	if ( ! defined('DS') ) { define('DS', DIRECTORY_SEPARATOR); }
	$Precode['Current']['dir'] = dirname(__FILE__);
	if (file_exists($Precode['Current']['dir'] . DS . "version.php")) {
		include_once($Precode['Current']['dir'] . DS . "version.php");
	}

	SDVA($Precode, array(
		'InitBase' => 1,
		'InitHighlight' => 0,
		'InitPresenter' => 0,
		'wwwdir' => '$PubDirUrl/precode', #requires moving js and css folders in pubdir/precode
		'jquery' => '/pub/jquery.min.js'
	));

/* USAGE
///////////////////////////////////////////////////
```showlines linereset=200 liststyle='upper-roman'
[@
@requires_authorization
def somefunc(param1='', param2=0):
    r'''A docstring'''
    if param1 > param2: # interesting
        print 'Gre\'ater'
    return (param2 - param1 + 1) or None

class SomeClass:
    pass
@]
```
///////////////////////////////////////////////////
*/

# Known issues
#  1. May not highlight on wikipage that starts with Markup.
#  2. There needs to be an empty line after Markup.

if(function_exists('Markup_e')) {
  # new format, no /e , IMPORTANT: pmwiki-2.2.58+ and php 5.5x
  Markup('```', '<split', "/^(`{3})([^\\`].*?)\\n(.*?)\\n\\1$/m",
      'CallbackPreCode');
} else {
  /* old format, for pmwiki < 2.2.58
     if using php 5.5x see pmwiki documentation as how to suppress
       deprecated preg_replace errors. */
  Markup('```', '<split', "/^(`{3})([^\\`].*?)\\n(.*?)\\n\\1$/em",
      "PreCode(\$pagename, PSS('$3'), PSS('$2'))");
}

function CallbackPreCode($m) {
  extract($GLOBALS["MarkupToHTML"]); # to get $pagename
  return PreCode($pagename, $m[3], $m[2]);
}

# Alternate markup (enable if needed)
#  modified from beautifier's markup
/* ( for pmwiki < 2.2.58 )
Markup("^```", "block", 
    "/^```(.*?)$KeepToken(\\d.*?)$KeepToken/e",
    "'<:block,1>'.PreCode(\$pagename, PSS('$2'), PSS('$1'))");
*/
/* alternate markup usage

```xml showlines linereset=200 liststyle='upper-roman' [@
  .... lines of code ....
  .... lines of code ....
@]

OR 

```xml showlines linereset=200 liststyle='upper-roman' [=
  .... lines of code ....
  .... lines of code ....
=]

*/

function _PreCode($pagename, $text, $args = '') {

	global $KeepToken, $KPV;
	$keepindex = trim(strip_tags($text), $KeepToken);
	$text = @trim($KPV[$keepindex], "\n"); $str_args = $args;

	$defaults = array(
		'class' => 'default',
		'theme' => 'default'
	);
	$_d = array(
		'_c' => 'c',
		'_ln' => 'linenum'
	);
	$args = ParseArgs($args);
	$args = array_merge($defaults, $args, $_d);

	$idt = isset($args['id']) ? $args['id'] : 'id'.substr(md5(microtime()),0,8);
	$id = $idt; $idc = $idt.$args['_c'];

	$lang = '';
	if ( isset($args[''][0]) ) { $lang = $args[''][0];
	} elseif( isset($args['lang']) ) { $lang = $args['lang']; }

	$showlines = ' nolinenum';
	if ( array_key_exists('', $args) && @in_array('showlines', $args['']) ) {
		_PreCodeLines($idt, $args); $showlines = ' linenum';
	}
	if ( @$args['showlines'] === '1') {
		_PreCodeLines($idt, $args); $showlines = ' linenum';
	}
	if (@$args['showlines'] === '0') { $showlines = ' nolinenum'; }

	$PreCode = compact(array(
		'str_args', 'args', 'id', 'idc', 'lang', 'text','showlines'
	));

	return $PreCode;

}

function _PreCodeLines($idt, &$args = array()) {

	global $HTMLFooterFmt, $HTMLStylesFmt;
	$_i = 'precode'.$idt;
	$resetrule = ''; $contentrule = '';
	static $_lnames = array();

	$liststyle = array(
		'disc','circle','square','decimal','decimal-leading-zero',
		'lower-roman','upper-roman','lower-greek','lower-latin','upper-latin',
		'armenian','georgian','lower-alpha','upper-alpha','none','inherit',
		'disclosure-open', 'disclosure-closed'
	);
	$pat['text1'] = '/[^\\w\\s~!@#$%*()=+:,.?]/';
	$pat['_lnc'] = '/[^\\w\\-\\_:]/';

	$_lnames['body']['linenum'] = 'linenum';
	$_lnames['pre']['linenum'] = "  pre code .numx:before { content: counter(linenum, decimal-leading-zero); counter-increment: linenum; }";

	if ( isset($args['classreset'])) {
		$_lnc = preg_replace($pat['_lnc'], '', $args['classreset']);
		@list($_lnc, $_lncn) = explode(':', $_lnc);
		$c_resetrule2 = "  pre.$_lnc code .numx:before { content: counter($_lnc, decimal-leading-zero); counter-increment: $_lnc; }";
		if(isset($_lncn)) { $_lnames[$_lnc]['_lncn'] = (int)$_lncn; }
		$_lnames['body'][$_lnc] = $_lnc.' '.@$_lnames[$_lnc]['_lncn'];
		$_lnames['pre'][$_lnc] = $c_resetrule2;
		$args['classreset'] = $_lnc;
	} else {
		$_lnc = $args['_ln'];
	}

	if ( array_key_exists('', $args) && @in_array('linereset', $args['']) ) {
		$resetrule = "  pre#$idt code { counter-reset: $_lnc; }";
	}
	if ( isset($args['linereset'])) {
		$n = (int)$args['linereset'];
		if( $n >= -100000 && $n <= 100000 ) {
			$n = "counter-reset: $_lnc $n;";
		}
		$resetrule = "  pre#$idt code { $n }";
	}

	if ( isset($args['liststyle'])) {
		$ls = $args['liststyle'];
		if( !in_array($ls, $liststyle)) {
			$ls = 'decimal-leading-zero';
		}
		$cn = "counter($_lnc, $ls)";
	}
	if ( isset($args['prefix'])) {
		$p1 = preg_replace($pat['text1'], '', $args['prefix']);
		if(isset($p1)) { $p1 = "'".substr($p1,0,8)."'"; }
	}
	if ( isset($args['suffix'])) {
		$p2 = preg_replace($pat['text1'], '', $args['suffix']);
		if(isset($p2)) { $p2 = "'".substr($p2,0,8)."'"; }
	}
	if ( isset($cn) || isset($p1) || isset($p2)) {
		$ls = 'content: '.@$p1.' '.@$cn.' '.@$p2.';';
	}

	if ( isset($args['step'])) {
		$_st = (int)$args['step'];
		if( $_st >= -100 && $_st <= 100 ) {
			$_st = 'counter-increment: '.@$_lnc.' '.@$_st.';';
		}
	}

	if ( isset($ls) || isset($_st)) {
		$contentrule = "  pre#$idt code .numx:before { ".@$ls.' '.@$_st.' }';
	}

	$_l1 = implode(" ", $_lnames['body']);
	$_l2 = implode("\n  ", $_lnames['pre']);
	$HTML3 = <<<EOF
  body { counter-reset: $_l1; }
  $_l2
EOF;
	$HTML4 = <<<EOF
$resetrule
$contentrule
EOF;

	$HTMLStylesFmt['PreCodeRule'] = $HTML3."\n";
	$HTMLStylesFmt[$_i] = $HTML4;

}

function PreCode($pagename, $text, $args = '') {
	if( ! function_exists('PreCode2')) {

		$PreCode = _PreCode($pagename, $text, $args);
		extract($PreCode, EXTR_OVERWRITE);

		static $highlights = array();
		include(dirname(__FILE__).'/adapter/adapter.base.php');

		$_id = " id='$id'"; $_idc = " id='$idc'";
		$_c = isset($args['classreset']) ? $args['classreset'].$args['_c'] : '';
		$_c1 = ' class="'.$args['theme'].$showlines.' hs '.@$args['classreset'].'"';
		$_c2 = ' class="'.$lang.' '.$_c.'"';
		return Keep('<pre'.$_id.$_c1.'><code'.$_idc.$_c2.'>'.$text.'</code></pre>');

	} else {
		return PreCode2($pagename, $text, $args);
	}
}
